==> pulp_container-1.0.0-py3-none-any.whl: pulp_container/tests/functional/api/test_crud_distributions.py <==
# coding=utf-8
"""Tests that CRUD distributions."""
import unittest

from itertools import permutations
from requests.exceptions import HTTPError

from pulp_smash import api, config, selectors, utils
from pulp_smash.pulp3.utils import gen_distribution

from pulp_container.tests.functional.constants import CONTAINER_DISTRIBUTION_PATH
from pulp_container.tests.functional.utils import set_up_module as setUpModule  # noqa:F401
from pulp_container.tests.functional.utils import skip_if


class CRUDContainerDistributionsTestCase(unittest.TestCase):
    """CRUD distributions."""

    @classmethod
    def setUpClass(cls):
        """Create class wide-variables."""
        cls.cfg = config.get_config()
        cls.client = api.Client(cls.cfg, api.json_handler)
        cls.distribution = {}

    def test_01_create_distribution(self):
        """Create a distribution."""
        body = gen_distribution()
        response_dict = self.client.post(
            CONTAINER_DISTRIBUTION_PATH, body
        )
        dist_task = self.client.get(response_dict['task'])
        distribution_href = dist_task['created_resources'][0]
        type(self).distribution = self.client.get(distribution_href)
        for key, val in body.items():
            with self.subTest(key=key):
                self.assertEqual(self.distribution[key], val)

    def test_02_create_same_name(self):
        """Try to create a second distribution with an identical name.

        See: `Pulp Smash #1055
        <https://github.com/PulpQE/pulp-smash/issues/1055>`_.
        """
        body = gen_distribution()
        body['name'] = self.distribution['name']
        with self.assertRaises(HTTPError):
            self.client.post(CONTAINER_DISTRIBUTION_PATH, body)

    @skip_if(bool, 'distribution', False)
    def test_02_read_distribution(self):
        """Read a distribution by its pulp_href."""
        distribution = self.client.get(self.distribution['pulp_href'])
        for key, val in self.distribution.items():
            with self.subTest(key=key):
                self.assertEqual(distribution[key], val)

    @skip_if(bool, 'distribution', False)
    def test_02_read_distribution_with_specific_fields(self):
        """Read a distribution by its href providing specific field list.

        Permutate field list to ensure different combinations on result.
        """
        if not selectors.bug_is_fixed(4599, self.cfg.pulp_version):
            raise unittest.SkipTest('Issue 4599 is not resolved')
        fields = ('base_path', 'name')
        for field_pair in permutations(fields, 2):
            # ex: field_pair = ('pulp_href', 'base_url)
            with self.subTest(field_pair=field_pair):
                distribution = self.client.get(
                    self.distribution['pulp_href'],
                    params={'fields': ','.join(field_pair)}
                )
                self.assertEqual(
                    sorted(field_pair), sorted(distribution.keys())
                )

    @skip_if(bool, 'distribution', False)
    def test_02_read_distribution_without_specific_fields(self):
        """Read a distribution by its href excluding specific fields."""
        if not selectors.bug_is_fixed(4599, self.cfg.pulp_version):
            raise unittest.SkipTest('Issue 4599 is not resolved')
        # requests doesn't allow the use of != in parameters.
        url = '{}?exclude_fields=base_path,name'.format(
            self.distribution['pulp_href']
        )
        distribution = self.client.get(url)
        response_fields = distribution.keys()
        self.assertNotIn('base_path', response_fields)
        self.assertNotIn('name', response_fields)

    @skip_if(bool, 'distribution', False)
    def test_02_read_distributions(self):
        """Read a distribution using query parameters.

        See: `Pulp #3082 <https://pulp.plan.io/issues/3082>`_
        """
        unique_params = (
            {'name': self.distribution['name']},
            {'base_path': self.distribution['base_path']}
        )
        for params in unique_params:
            with self.subTest(params=params):
                page = self.client.get(CONTAINER_DISTRIBUTION_PATH, params=params)
                self.assertEqual(len(page['results']), 1)
                for key, val in self.distribution.items():
                    with self.subTest(key=key):
                        self.assertEqual(page['results'][0][key], val)

    @skip_if(bool, 'distribution', False)
    def test_03_partially_update(self):
        """Update a distribution using HTTP PATCH."""
        body = gen_distribution()
        self.client.patch(self.distribution['pulp_href'], body)
        type(self).distribution = self.client.get(self.distribution['pulp_href'])
        for key, val in body.items():
            with self.subTest(key=key):
                self.assertEqual(self.distribution[key], val)

    @skip_if(bool, 'distribution', False)
    def test_04_fully_update(self):
        """Update a distribution using HTTP PUT."""
        body = gen_distribution()
        self.client.put(self.distribution['pulp_href'], body)
        type(self).distribution = self.client.get(self.distribution['pulp_href'])
        for key, val in body.items():
            with self.subTest(key=key):
                self.assertEqual(self.distribution[key], val)

    @skip_if(bool, 'distribution', False)
    def test_05_delete(self):
        """Delete a distribution."""
        self.client.delete(self.distribution['pulp_href'])
        with self.assertRaises(HTTPError):
            self.client.get(self.distribution['pulp_href'])

    def test_negative_create_distribution_with_invalid_parameter(self):
        """Attempt to create distribution passing invalid parameter.

        Assert response returns an error 400 including ["Unexpected field"].
        """
        response = api.Client(self.cfg, api.echo_handler).post(
            CONTAINER_DISTRIBUTION_PATH, gen_distribution(foo='bar')
        )
        assert response.status_code == 400
        assert response.json()['foo'] == ['Unexpected field']


class DistributionBasePathTestCase(unittest.TestCase):
    """Test possible values for ``base_path`` on a distribution.

    This test targets the following issues:

    * `Pulp #2987 <https://pulp.plan.io/issues/2987>`_
    * `Pulp #3412 <https://pulp.plan.io/issues/3412>`_
    * `Pulp Smash #906 <https://github.com/PulpQE/pulp-smash/issues/906>`_
    * `Pulp Smash #956 <https://github.com/PulpQE/pulp-smash/issues/956>`_
    """

    @classmethod
    def setUpClass(cls):
        """Create class-wide variables."""
        cls.cfg = config.get_config()
        cls.client = api.Client(cls.cfg, api.json_handler)
        body = gen_distribution()
        body['base_path'] = body['base_path'].replace('-', '/')
        response_dict = cls.client.post(CONTAINER_DISTRIBUTION_PATH, body)
        dist_task = cls.client.get(response_dict['task'])
        distribution_href = dist_task['created_resources'][0]
        cls.distribution = cls.client.get(distribution_href)

    @classmethod
    def tearDownClass(cls):
        """Clean up resources."""
        cls.client.delete(cls.distribution['pulp_href'])

    def test_spaces(self):
        """Test that spaces can not be part of ``base_path``."""
        self.try_create_distribution(base_path=utils.uuid4().replace('-', ' '))
        self.try_update_distribution(base_path=utils.uuid4().replace('-', ' '))

    def test_begin_slash(self):
        """Test that slash cannot be in the begin of ``base_path``."""
        self.try_create_distribution(base_path='/' + utils.uuid4())
        self.try_update_distribution(base_path='/' + utils.uuid4())

    def test_end_slash(self):
        """Test that slash cannot be in the end of ``base_path``."""
        self.try_create_distribution(base_path=utils.uuid4() + '/')
        self.try_update_distribution(base_path=utils.uuid4() + '/')

    def test_unique_base_path(self):
        """Test that ``base_path`` can not be duplicated."""
        self.try_create_distribution(base_path=self.distribution['base_path'])

    def try_create_distribution(self, **kwargs):
        """Unsuccessfully create a distribution.

        Merge the given kwargs into the body of the request.
        """
        body = gen_distribution()
        body.update(kwargs)
        with self.assertRaises(HTTPError):
            self.client.post(CONTAINER_DISTRIBUTION_PATH, body)

    def try_update_distribution(self, **kwargs):
        """Unsuccessfully update a distribution with HTTP PATCH.

        Use the given kwargs as the body of the request.
        """
        with self.assertRaises(HTTPError):
            self.client.patch(self.distribution['pulp_href'], kwargs)

==> pulp_container-1.1.0-py3-none-any.whl: pulp_container/tests/functional/api/test_crud_distributions.py <==
# coding=utf-8
"""Tests that CRUD distributions."""
import unittest

from itertools import permutations
from requests.exceptions import HTTPError

from pulp_smash import api, config, selectors, utils
from pulp_smash.pulp3.utils import gen_distribution

from pulp_container.tests.functional.constants import CONTAINER_DISTRIBUTION_PATH
from pulp_container.tests.functional.utils import set_up_module as setUpModule  # noqa:F401
from pulp_container.tests.functional.utils import skip_if


class CRUDContainerDistributionsTestCase(unittest.TestCase):
    """CRUD distributions."""

    @classmethod
    def setUpClass(cls):
        """Create class wide-variables."""
        cls.cfg = config.get_config()
        cls.client = api.Client(cls.cfg, api.json_handler)
        cls.distribution = {}

    def test_01_create_distribution(self):
        """Create a distribution."""
        body = gen_distribution()
        response_dict = self.client.post(
            CONTAINER_DISTRIBUTION_PATH, body
        )
        dist_task = self.client.get(response_dict['task'])
        distribution_href = dist_task['created_resources'][0]
        type(self).distribution = self.client.get(distribution_href)
        for key, val in body.items():
            with self.subTest(key=key):
                self.assertEqual(self.distribution[key], val)

    def test_02_create_same_name(self):
        """Try to create a second distribution with an identical name.

        See: `Pulp Smash #1055
        <https://github.com/pulp/pulp-smash/issues/1055>`_.
        """
        body = gen_distribution()
        body['name'] = self.distribution['name']
        with self.assertRaises(HTTPError):
            self.client.post(CONTAINER_DISTRIBUTION_PATH, body)

    @skip_if(bool, 'distribution', False)
    def test_02_read_distribution(self):
        """Read a distribution by its pulp_href."""
        distribution = self.client.get(self.distribution['pulp_href'])
        for key, val in self.distribution.items():
            with self.subTest(key=key):
                self.assertEqual(distribution[key], val)

    @skip_if(bool, 'distribution', False)
    def test_02_read_distribution_with_specific_fields(self):
        """Read a distribution by its href providing specific field list.

        Permutate field list to ensure different combinations on result.
        """
        if not selectors.bug_is_fixed(4599, self.cfg.pulp_version):
            raise unittest.SkipTest('Issue 4599 is not resolved')
        fields = ('base_path', 'name')
        for field_pair in permutations(fields, 2):
            # ex: field_pair = ('pulp_href', 'base_url)
            with self.subTest(field_pair=field_pair):
                distribution = self.client.get(
                    self.distribution['pulp_href'],
                    params={'fields': ','.join(field_pair)}
                )
                self.assertEqual(
                    sorted(field_pair), sorted(distribution.keys())
                )

    @skip_if(bool, 'distribution', False)
    def test_02_read_distribution_without_specific_fields(self):
        """Read a distribution by its href excluding specific fields."""
        if not selectors.bug_is_fixed(4599, self.cfg.pulp_version):
            raise unittest.SkipTest('Issue 4599 is not resolved')
        # requests doesn't allow the use of != in parameters.
        url = '{}?exclude_fields=base_path,name'.format(
            self.distribution['pulp_href']
        )
        distribution = self.client.get(url)
        response_fields = distribution.keys()
        self.assertNotIn('base_path', response_fields)
        self.assertNotIn('name', response_fields)

    @skip_if(bool, 'distribution', False)
    def test_02_read_distributions(self):
        """Read a distribution using query parameters.

        See: `Pulp #3082 <https://pulp.plan.io/issues/3082>`_
        """
        unique_params = (
            {'name': self.distribution['name']},
            {'base_path': self.distribution['base_path']}
        )
        for params in unique_params:
            with self.subTest(params=params):
                page = self.client.get(CONTAINER_DISTRIBUTION_PATH, params=params)
                self.assertEqual(len(page['results']), 1)
                for key, val in self.distribution.items():
                    with self.subTest(key=key):
                        self.assertEqual(page['results'][0][key], val)

    @skip_if(bool, 'distribution', False)
    def test_03_partially_update(self):
        """Update a distribution using HTTP PATCH."""
        body = gen_distribution()
        self.client.patch(self.distribution['pulp_href'], body)
        type(self).distribution = self.client.get(self.distribution['pulp_href'])
        for key, val in body.items():
            with self.subTest(key=key):
                self.assertEqual(self.distribution[key], val)

    @skip_if(bool, 'distribution', False)
    def test_04_fully_update(self):
        """Update a distribution using HTTP PUT."""
        body = gen_distribution()
        self.client.put(self.distribution['pulp_href'], body)
        type(self).distribution = self.client.get(self.distribution['pulp_href'])
        for key, val in body.items():
            with self.subTest(key=key):
                self.assertEqual(self.distribution[key], val)

    @skip_if(bool, 'distribution', False)
    def test_05_delete(self):
        """Delete a distribution."""
        self.client.delete(self.distribution['pulp_href'])
        with self.assertRaises(HTTPError):
            self.client.get(self.distribution['pulp_href'])

    def test_negative_create_distribution_with_invalid_parameter(self):
        """Attempt to create distribution passing invalid parameter.

        Assert response returns an error 400 including ["Unexpected field"].
        """
        response = api.Client(self.cfg, api.echo_handler).post(
            CONTAINER_DISTRIBUTION_PATH, gen_distribution(foo='bar')
        )
        assert response.status_code == 400
        assert response.json()['foo'] == ['Unexpected field']


class DistributionBasePathTestCase(unittest.TestCase):
    """Test possible values for ``base_path`` on a distribution.

    This test targets the following issues:

    * `Pulp #2987 <https://pulp.plan.io/issues/2987>`_
    * `Pulp #3412 <https://pulp.plan.io/issues/3412>`_
    * `Pulp Smash #906 <https://github.com/pulp/pulp-smash/issues/906>`_
    * `Pulp Smash #956 <https://github.com/pulp/pulp-smash/issues/956>`_
    """

    @classmethod
    def setUpClass(cls):
        """Create class-wide variables."""
        cls.cfg = config.get_config()
        cls.client = api.Client(cls.cfg, api.json_handler)
        body = gen_distribution()
        body['base_path'] = body['base_path'].replace('-', '/')
        response_dict = cls.client.post(CONTAINER_DISTRIBUTION_PATH, body)
        dist_task = cls.client.get(response_dict['task'])
        distribution_href = dist_task['created_resources'][0]
        cls.distribution = cls.client.get(distribution_href)

    @classmethod
    def tearDownClass(cls):
        """Clean up resources."""
        cls.client.delete(cls.distribution['pulp_href'])

    def test_spaces(self):
        """Test that spaces can not be part of ``base_path``."""
        self.try_create_distribution(base_path=utils.uuid4().replace('-', ' '))
        self.try_update_distribution(base_path=utils.uuid4().replace('-', ' '))

    def test_begin_slash(self):
        """Test that slash cannot be in the begin of ``base_path``."""
        self.try_create_distribution(base_path='/' + utils.uuid4())
        self.try_update_distribution(base_path='/' + utils.uuid4())

    def test_end_slash(self):
        """Test that slash cannot be in the end of ``base_path``."""
        self.try_create_distribution(base_path=utils.uuid4() + '/')
        self.try_update_distribution(base_path=utils.uuid4() + '/')

    def test_unique_base_path(self):
        """Test that ``base_path`` can not be duplicated."""
        self.try_create_distribution(base_path=self.distribution['base_path'])

    def try_create_distribution(self, **kwargs):
        """Unsuccessfully create a distribution.

        Merge the given kwargs into the body of the request.
        """
        body = gen_distribution()
        body.update(kwargs)
        with self.assertRaises(HTTPError):
            self.client.post(CONTAINER_DISTRIBUTION_PATH, body)

    def try_update_distribution(self, **kwargs):
        """Unsuccessfully update a distribution with HTTP PATCH.

        Use the given kwargs as the body of the request.
        """
        with self.assertRaises(HTTPError):
            self.client.patch(self.distribution['pulp_href'], kwargs)
